@@ -6,6 +6,7 @@ This module performs the return type inference, according to symbolic types,
 
 from pythran.analyses import LazynessAnalysis, StrictAliases, YieldPoints
 from pythran.analyses import LocalNodeDeclarations, Immediates, RangeValues
+from pythran.analyses import Ancestors
 from pythran.config import cfg
 from pythran.cxxtypes import TypeBuilder, ordered_set
 from pythran.intrinsic import UserFunction, Class
@@ -22,8 +23,6 @@ from itertools import islice
 from copy import deepcopy
 
 
-MAX_ELTS = 8
-
 class UnboundableRValue(Exception):
     pass
 
@@ -34,6 +33,9 @@ class Types(ModuleAnalysis):
 
     def __init__(self):
 
+        self.max_seq_size = cfg.getint('typing',
+                                       'max_heterogeneous_sequence_size')
+
         class TypeResult(dict):
             def __init__(self):
                 self.builder = TypeBuilder()
@@ -51,7 +53,7 @@ class Types(ModuleAnalysis):
         self.current_global_declarations = dict()
         self.max_recompute = 1  # max number of use to be lazy
         ModuleAnalysis.__init__(self, Reorder, StrictAliases, LazynessAnalysis,
-                                Immediates, RangeValues)
+                                Immediates, RangeValues, Ancestors)
         self.curr_locals_declaration = None
 
     def combined(self, *types):
@@ -492,8 +494,24 @@ class Types(ModuleAnalysis):
             self.result[node] = self.builder.NamedType(
                 'pythonic::types::slice')
 
+    def stores_to(self, node):
+        ancestors = self.ancestors[node] + (node,)
+        stmt_indices = [i for i, n in enumerate(ancestors)
+                        if isinstance(n, (ast.Assign, ast.For))]
+        if not stmt_indices:
+            return True
+
+        stmt_index = stmt_indices[-1]
+
+        if isinstance(ancestors[stmt_index], ast.Assign):
+            return ancestors[stmt_index + 1] is ancestors[stmt_index].value
+        else:
+            return ancestors[stmt_index + 1] is not ancestors[stmt_index].target
+
     def visit_Subscript(self, node):
         self.visit(node.value)
+        if self.stores_to(node):
+            self.result[node.value] = self.builder.AddConst(self.result[node.value])
         # type of a[1:2, 3, 4:1] is the type of: declval(a)(slice, long, slice)
         if isextslice(node.slice):
             self.visit(node.slice)
@@ -550,7 +568,7 @@ class Types(ModuleAnalysis):
         """ Define list type from all elements type (or empty_list type). """
         self.generic_visit(node)
         if node.elts:
-            for elt in node.elts[:MAX_ELTS]:
+            for elt in node.elts[:self.max_seq_size]:
                 self.update_type(node, self.builder.ListType, self.result[elt])
         else:
             self.update_type(node,
@@ -561,7 +579,7 @@ class Types(ModuleAnalysis):
         """ Define set type from all elements type (or empty_set type). """
         self.generic_visit(node)
         if node.elts:
-            for elt in node.elts[:MAX_ELTS]:
+            for elt in node.elts[:self.max_seq_size]:
                 self.update_type(node, self.builder.SetType, self.result[elt])
         else:
             self.update_type(node, self.builder.NamedType,
@@ -571,7 +589,8 @@ class Types(ModuleAnalysis):
         """ Define set type from all elements type (or empty_dict type). """
         self.generic_visit(node)
         if node.keys:
-            for key, value in islice(zip(node.keys, node.values), MAX_ELTS):
+            for key, value in islice(zip(node.keys, node.values),
+                                     self.max_seq_size):
                 self.update_type(node,
                                  self.builder.DictType,
                                  self.result[key],
